function b = dec_tetro_coeff(a,J,num_cov)
%
% filterbank decomposition algorithm with coefficients display
%
% INPUT:
% a -        input image
% J -        number of decomposition level
% num_cov -  number of admissible coverings
%
% OUTPUT:
% b: cell array of size 3x(J+1) with tetrolet coefficients
%
%
% CHANGELOG
%
%  2009-10-01 (jens.krommweh@uni-due.de)


% ### initialisation ###
global COV_DISTRIB
[WW,P] = init_tetro(num_cov);

[N1,N2] = size(a);

coef_cell = cell(J+1,3);  % first entry contains lowpass part, further entries the wave coefficients
distr = zeros(J,num_cov); % distribution of choosen covering
dis = zeros(1,num_cov);   % distribution (only needed for relaxed version)
coef_cell{1,1} = a;       % initialization with input image
b = zeros(size(a));

% #####################
% ### Decomposition ###
% #####################

for zz = 1:J
    N1 = N1/2; N2 = N2/2;
    coef_high = cell(N1/2,N2/2);
    lowpass = zeros(N1);
    % subband matrices for coefficient display
    subband1 = zeros(N1);
    subband2 = zeros(N1);
    subband3 = zeros(N1);
    
    covering = zeros(N1/2,N2/2); % map of choosen tetromino coverings
        
for ii = 1:N1/2
    for jj = 1:N2/2

        % 4x4-blocking
        B = coef_cell{1,1}(4*ii-3:4*ii,4*jj-3:4*jj);

        pixel_matrix = zeros(size(P));
        pixel_matrix(:,1) = B(:);
        pixel_matrix = cumsum(pixel_matrix,2);
        
        % find optimal tetrolet decomposition
        [r, wave_min, permut] = opt_wave10(pixel_matrix(:)); % r: lowpass part, wave_min: wavelets, permut: number of covering

        % store wave coefficients with corresponding permutation vector
        coef_high{ii,jj} = wave_min; % 12er vector
        covering(ii,jj) = permut;    % scalar
        % distribution
        dis(permut) = dis(permut) + 1;
        % store lowpass part
        lowpass(2*ii-1:2*ii,2*jj-1:2*jj) = reshape(r,2,2);
        % coefficient display
        w1 = wave_min(1:3:end); w2 = wave_min(2:3:end); w3 = wave_min(3:3:end); 
        subband1(2*ii-1:2*ii,2*jj-1:2*jj) = reshape(w1,2,2);
        subband2(2*ii-1:2*ii,2*jj-1:2*jj) = reshape(w2,2,2);
        subband3(2*ii-1:2*ii,2*jj-1:2*jj) = reshape(w3,2,2);
    end
end
    coef_cell{1,1} = lowpass;  % lowpass-part
    coef_cell{zz+1,1} = coef_high;
    coef_cell{zz+1,2} = covering;
    coef_cell{zz+1,3} = [lowpass subband1; subband2 subband3];

    % display coeffients    
    subband = [wcodemat(lowpass,2*size(a,1)) wcodemat(subband1,2*size(a,1)); ...
               wcodemat(subband2,2*size(a,1)) wcodemat(subband3,2*size(a,1))]; % coefficient matrix
    figure, imagesc(subband), colormap(gray), axis square;
    b(1:2*N1,1:2*N2) = subband;
    figure, imagesc(b), colormap(gray), axis square;
    
    filename = ['transformcoeff_', num2str(zz)];
    print(gcf, '-depsc2', filename);

    %covering, %figure, hist(covering, 1:num_cov); % display covering distribution
    distr(zz,:) = sum(histc(covering, 1:num_cov,1),2);
end

COV_DISTRIB = sum(distr); % covering distribution ( = dis)
%figure, bar(COV_DISTRIB); % display covering distribution

b = coef_cell;