function demo_tetro
%
% image approximation with tetrolet filter bank which is outlined in:
%
%       J. Krommweh, Tetrolet Transform: A New Adaptive Haar Wavelet
%       Algorithm for Sparse Image Representation
%
% INPUT: a - original image (square and dyadic size)
%        N - coefficients after shrinkage (best N-term approximation) 
%        J - number of decomposition levels (default: log2(size(a,1))-1 )
%        num_cov - number of admissible tetromino coverings
%                  admissible value: 16, 32, 64, 117 (default)
%
% OUTPUT: reconstructed image
%
% 2009-02-20 (jens.krommweh@uni-due.de)

clc;
close all;

fprintf('\n-----------   Welcome to Matlab Tetrolet Transform !   -----------\n\n');

% ################
% ### I) input ###
% ################

a = double(imread('lena256.png')); % square and dyadic image

N = 2048;
J = 5;% log2(size(a,1))-1;            % level
num_cov = 117;                     % use 117 or 16


% ###############################
% ### II) tetrolet filterbank ###
% ###############################

global COV_DISTRIB

% decomposition
tic; b1 = dec_tetro(a,J,num_cov); tdec=toc;
% shrinkage
c1 = thresh_cell(b1,N);
% reconstruction
tic; b1 = rec_tetro(c1); trec = toc;
p1 = psnr(a,b1);

% compute storage costs
E = entr(COV_DISTRIB, size(a,1));
storage_cost = E + cost_wave(N, size(a,1));

fprintf(' \nTETROLET TRANSFORM \n');
fprintf('------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N)]);
fprintf(1,['\nPSNR: ',num2str(p1)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% #########################################################
% ### III) comparison with biorthogonal 9/7 filter bank ###
% #########################################################

% decomposition
tic; b3 = waveletcdf97(a, J+1); tdec=toc;
% shrinkage
[c3,N3] = thresh_matrix(b3, J+1, N);
% reconstruction
tic; b3 = waveletcdf97(c3, -(J+1)); trec=toc;
p3 = psnr(a,b3);

% compute storage costs
storage_cost3 = cost_wave(N3, size(a,1));

fprintf(' \nBiorthogonal 9-7 WAVELET TRANSFORM \n');
fprintf('----------------------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N3)]);
fprintf(1,['\nPSNR: ',num2str(p3)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost3),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% #########################
% ### IV) print figures ###
% #########################
figure,
SUBPLOT(1,3,1), imagesc(a), axis square, colormap(gray), title('Input');
SUBPLOT(1,3,2), imagesc(b1), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (Tetrolet), PSNR %.2f',N,p1));
SUBPLOT(1,3,3), imagesc(b3), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (biorth 9-7), PSNR %.2f',N3,p3));
set(gcf, 'Units', 'inches'); set(gcf, 'Position', [1, 3.5, 12, 4])

% ########################################
% ### V) modified tetrolet transforms ###
% ########################################

% ### V.1) tetrolet 16 ###

% decomposition
tic; b1 = dec_tetro(a,J,16); tdec=toc;
% shrinkage
c1 = thresh_cell(b1,N);
% reconstruction
tic; d1 = rec_tetro(c1); trec=toc;
p1 = psnr(a,d1);

% compute storage costs
E = entr(COV_DISTRIB, size(a,1));
storage_cost = E + cost_wave(N, size(a,1));

fprintf(' \nMODIFIED TETROLET TRANSFORM "tetrolet 16" \n');
fprintf('-----------------------------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N)]);
fprintf(1,['\nPSNR: ',num2str(p1)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% ### V.2) tetrolet relaxed ###

% decomposition
tic; b1 = dec_tetro_rel(a,J,num_cov,25); tdec=toc;
% shrinkage
c1 = thresh_cell(b1,N);
% reconstruction
tic; d2 = rec_tetro(c1); trec=toc;
p2 = psnr(a,d2);

% compute storage costs
E = entr(COV_DISTRIB, size(a,1));
storage_cost = E + cost_wave(N, size(a,1));

fprintf(' \nMODIFIED TETROLET TRANSFORM "tetrolet relaxed" \n');
fprintf('----------------------------------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N)]);
fprintf(1,['\nPSNR: ',num2str(p2)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% ### V.3) tetrolet edge ###

% decomposition
tic; b1 = dec_tetro_edge(a,J,num_cov); tdec=toc;
% shrinkage
c1 = thresh_cell(b1,N);
% reconstruction
tic; d3 = rec_tetro(c1); trec=toc;
p3 = psnr(a,d3);

% compute storage costs
E = entr(COV_DISTRIB, size(a,1));
storage_cost = E + cost_wave(N, size(a,1));

fprintf(' \nMODIFIED TETROLET TRANSFORM "tetrolet edge" \n');
fprintf('-------------------------------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N)]);
fprintf(1,['\nPSNR: ',num2str(p3)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% ### V.4) tetrolet 16 rel edge ###

% decomposition
tic; b1 = dec_tetro_rel_edge(a,J,16,25); tdec=toc;
% shrinkage
c1 = thresh_cell(b1,N);
% reconstruction
tic; d4 = rec_tetro(c1); trec=toc;
p4 = psnr(a,d4);

% compute storage costs
E = entr(COV_DISTRIB, size(a,1));
storage_cost = E + cost_wave(N, size(a,1));

fprintf(' \nMODIFIED TETROLET TRANSFORM "tetrolet 16 rel edge" \n');
fprintf('--------------------------------------------------');
fprintf(1,['\nCoefficients after shrinkage: ',num2str(N)]);
fprintf(1,['\nPSNR: ',num2str(p4)]);
fprintf(1,['\nStorage costs: ',num2str(storage_cost),' bpp']);
fprintf(1,['\nDec time: ',num2str(tdec),' sec, Rec time: ',num2str(trec),' sec\n\n']);

% #########################
% ### VI) print figures ###
% #########################
figure,
SUBPLOT(2,2,1), imagesc(d1), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (Tetrolet 16), PSNR %.2f',N,p1));
SUBPLOT(2,2,2), imagesc(d2), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (Tetrolet rel), PSNR %.2f',N,p2));
SUBPLOT(2,2,3), imagesc(d3), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (Tetrolet edge), PSNR %.2f',N,p3));
SUBPLOT(2,2,4), imagesc(d4), axis square, colormap(gray), ...
    title(sprintf('rec with %.d coeff (Tetrolet 16 rel edge), PSNR %.2f',N,p4));
set(gcf, 'Units', 'inches'); set(gcf, 'Position', [2, 3, 10, 8])
